@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 # NanoB2B::NER
-# (Last Updated $Id: NER.pm,v 1.1 2017/04/03 16:52:33 charityml Exp $)
+# (Last Updated $Id: NER.pm,v 0.09 2018/01/18 16:33:33 charityml Exp $)
 #
 # Perl module that turns labeled text lines into 
 # ARFF files based on specified features
@@ -99,6 +99,7 @@ use NanoB2B::NER::Metaman;
 use NanoB2B::NER::Arffman;
 use NanoB2B::NER::Wekaman;
 use NanoB2B::NER::Avgman;
+use NanoB2B::NER::Modelman;
 
 #the instances of the modules (named 'boy' because they are expendable and constantly changing - like Robin, BOY Wonder to BatMAN)
 my $uniSub;
@@ -106,8 +107,9 @@ my $metaboy;
 my $arffboy;
 my $wekaboy;
 my $avgboy;
+my $modelboy;
 
-our $VERSION = '0.01';
+our $VERSION = '0.09';
 
 #option variables
 my $debug = 0;
@@ -127,7 +129,7 @@ found from pre-annotated articles
 
 =head1 VERSION
 
-Version 0.01
+Version 0.09
 
 =head1 INITIALIZING THE MODULE
 
@@ -155,7 +157,7 @@ sub new {
     return undef if(ref $class);
     my $params = shift;
 
-    #bless this object
+    #bless this object - amen
     bless $self, $class;
     $self->_init($params);
 
@@ -195,6 +197,7 @@ sub _init {
     my $opt_wekatype 	= $params->{'weka_type'};
     my $opt_wekasize 	= $params->{'weka_size'};
 	my $opt_metamaparguments = $params->{'metamap_arguments'};
+	my $opt_model	    = $params->{'model'};
 
 
     #set the global variables
@@ -299,6 +302,12 @@ sub _init {
 		$self->{metamap_arguments} = "-q";
 	}
 
+	if(defined $opt_model){
+		$self->{model} = $opt_model;
+	}else{
+		$self->{model} = 0;
+	}
+
 	#error handling?
 	if(defined $opt_file and $self->{fileIndex} > 1){
 		print("***ERROR: Cannot have an index value for a single file!***\n");
@@ -402,30 +411,49 @@ sub nerByFile{
 			$arffboy->arff_file($tag);
 
 			#weka the file
-			$uniSub->printColorDebug("bold yellow", "---    WEKA    ---\n");
-				#define wekaboy with the parameters
-				my %paramsw = ();
-				$paramsw{'directory'} = $self->{program_dir};
-				$paramsw{'type'} = $self->{weka_type};
-				$paramsw{'weka_size'} = $self->{weka_size};
-				$paramsw{'features'} = $self->{features};
-				$paramsw{'buckets'} = $self->{bucketsNum};
-				$paramsw{'debug'} = $debug;
-				$wekaboy = NanoB2B::NER::Wekaman->new(\%paramsw);
-			$wekaboy->weka_file($tag);
+			if($self->{bucketsNum} > 1){
+				$uniSub->printColorDebug("bold yellow", "---    WEKA    ---\n");
+					#define wekaboy with the parameters
+					my %paramsw = ();
+					$paramsw{'directory'} = $self->{program_dir};
+					$paramsw{'type'} = $self->{weka_type};
+					$paramsw{'weka_size'} = $self->{weka_size};
+					$paramsw{'features'} = $self->{features};
+					$paramsw{'buckets'} = $self->{bucketsNum};
+					$paramsw{'debug'} = $debug;
+					$wekaboy = NanoB2B::NER::Wekaman->new(\%paramsw);
+				$wekaboy->weka_file($tag);
+			}
 
-			#average the set
-			$uniSub->printColorDebug("bold green", "---    AVG    ---\n");
-				#define avgboy with the parameters
-				my %paramsa = ();
-				$paramsa{'directory'} = $self->{program_dir};
-					my @a = split(/\./, $self->{weka_type});
-				$paramsa{'weka_dir'} = $a[$#a];
-				$paramsa{'features'} = $self->{features};
-				$paramsa{'buckets'} = $self->{bucketsNum};
-				$paramsa{'debug'} = $debug;
-				$avgboy = NanoB2B::NER::Avgman->new(\%paramsa);	
-			$avgboy->avg_file($tag);		
+			#model the file
+			if($self->{model}){
+				$uniSub->printColorDebug("bold red", "---  MODEL  ---\n");
+					my %paramsmo = ();
+					$paramsmo{'directory'} = $self->{program_dir};
+					$paramsmo{'type'} = $self->{weka_type};
+					$paramsmo{'weka_size'} = $self->{weka_size};
+					$paramsmo{'features'} = $self->{features};
+					$paramsmo{'buckets'} = $self->{bucketsNum};
+					$paramsmo{'debug'} = $debug;
+					$modelboy = NanoB2B::NER::Modelman->new(\%paramsmo);
+				$modelboy->make_model_file($tag);
+			}
+
+			#average the file
+			if($self->{bucketsNum} > 1){
+				$uniSub->printColorDebug("bold green", "---    AVG    ---\n");
+					#define avgboy with the parameters
+					my %paramsa = ();
+					$paramsa{'directory'} = $self->{program_dir};
+						my @a = split(/\./, $self->{weka_type});
+					$paramsa{'weka_dir'} = $a[$#a];
+					$paramsa{'features'} = $self->{features};
+					$paramsa{'buckets'} = $self->{bucketsNum};
+					$paramsa{'debug'} = $debug;
+					$avgboy = NanoB2B::NER::Avgman->new(\%paramsa);	
+				$avgboy->avg_file($tag);	
+			}
+				
 
 			$uniSub->printColorDebug("on_blue", "##         FINISHED #$a - $tag!        ##");
 		}
@@ -467,30 +495,49 @@ sub nerByFile{
 		$arffboy->arff_file($tag);
 
 		#weka the file
-		$uniSub->printColorDebug("bold yellow", "---    WEKA    ---\n");
-			#define wekaboy with the parameters
-			my %paramsw = ();
-			$paramsw{'directory'} = $self->{program_dir};
-			$paramsw{'type'} = $self->{weka_type};
-			$paramsw{'weka_size'} = $self->{weka_size};
-			$paramsw{'features'} = $self->{features};
-			$paramsw{'buckets'} = $self->{bucketsNum};
-			$paramsw{'debug'} = $debug;
-			$wekaboy = NanoB2B::NER::Wekaman->new(\%paramsw);
-		$wekaboy->weka_file($tag);
+		if($self->{bucketsNum} > 1){
+			$uniSub->printColorDebug("bold yellow", "---    WEKA    ---\n");
+				#define wekaboy with the parameters
+				my %paramsw = ();
+				$paramsw{'directory'} = $self->{program_dir};
+				$paramsw{'type'} = $self->{weka_type};
+				$paramsw{'weka_size'} = $self->{weka_size};
+				$paramsw{'features'} = $self->{features};
+				$paramsw{'buckets'} = $self->{bucketsNum};
+				$paramsw{'debug'} = $debug;
+				$wekaboy = NanoB2B::NER::Wekaman->new(\%paramsw);
+			$wekaboy->weka_file($tag);
+		}
 
-		#average the set
-		$uniSub->printColorDebug("bold green", "---    AVG    ---\n");
-			#define avgboy with the parameters
-			my %paramsa = ();
-			$paramsa{'directory'} = $self->{program_dir};
-				my @a = split(/\./, $self->{weka_type});
-			$paramsa{'weka_dir'} = $a[$#a];
-			$paramsa{'features'} = $self->{features};
-			$paramsa{'buckets'} = $self->{bucketsNum};
-			$paramsa{'debug'} = $debug;
-			$avgboy = NanoB2B::NER::Avgman->new(\%paramsa);	
-		$avgboy->avg_file($tag);		
+		#model the file
+		if($self->{model}){
+			$uniSub->printColorDebug("bold red", "---  MODEL  ---\n");
+				my %paramsmo = ();
+				$paramsmo{'directory'} = $self->{program_dir};
+				$paramsmo{'type'} = $self->{weka_type};
+				$paramsmo{'weka_size'} = $self->{weka_size};
+				$paramsmo{'features'} = $self->{features};
+				$paramsmo{'buckets'} = $self->{bucketsNum};
+				$paramsmo{'debug'} = $debug;
+				$modelboy = NanoB2B::NER::Modelman->new(\%paramsmo);
+			$modelboy->make_model_file($tag);
+		}
+
+		#average the file
+		if($self->{bucketsNum} > 1){
+			$uniSub->printColorDebug("bold green", "---    AVG    ---\n");
+				#define avgboy with the parameters
+				my %paramsa = ();
+				$paramsa{'directory'} = $self->{program_dir};
+					my @a = split(/\./, $self->{weka_type});
+				$paramsa{'weka_dir'} = $a[$#a];
+				$paramsa{'features'} = $self->{features};
+				$paramsa{'buckets'} = $self->{bucketsNum};
+				$paramsa{'debug'} = $debug;
+				$avgboy = NanoB2B::NER::Avgman->new(\%paramsa);	
+			$avgboy->avg_file($tag);	
+		}
+			
 
 		$uniSub->printColorDebug("on_blue", "##         FINISHED #$tag!        ##");
 	}					
@@ -537,11 +584,20 @@ sub nerByMethod{
 	$self->arffSet();
 
 	#weka the files
-	$self->wekaSet();
+	if($self->{bucketsNum} > 1){
+		$self->wekaSet();
+	}
+	
+
+	if($self->{model}){
+		$self->modelSet();
+	}
 
 	#average the files
-	$self->avgSet();
-}
+	if($self->{bucketsNum} > 1){
+		$self->avgSet();
+	}
+}	
 
 =head3 metaSet
 
@@ -754,6 +810,74 @@ sub wekaSet{
 	}
 }
 
+=head3 modelSet
+
+description:
+
+  Creates WEKA models from the training ARFF files
+
+input:
+
+ None
+
+output:
+
+ WEKA model files for every file with training ARFF files
+
+example:
+
+  use NanoB2B::NER;
+  my %params =  ();
+  	$params{'dir'} = "my_directory";
+  	$params{'features'} = "ortho morph text pos cui sem";
+
+  my $nner = new NanoB2B::NER(\%params);
+  $nner->modelSet();
+
+=cut
+sub modelSet{
+	my $self = shift;
+
+	#open the directory	 
+	opendir (my $DIR, $self->{program_dir}) or die $!;							
+	my @tags = grep { $_ ne '.' and $_ ne '..' and substr($_, 0, 1) ne '_'} readdir $DIR;	#get each file from the directory
+	my $totalTags = @tags;
+
+	#if only one file
+	if($self->{program_file}){
+		$totalTags = 1;
+	}
+
+	#sort by size?
+	if($self->{sortSize}){
+		@tags = sortBySize($self, \@tags);
+	}
+
+	#if only one file reduce it to the one
+	if(defined $self->{program_file}){
+		@tags = ($self->{program_file});
+	}
+
+	#define wekaboy with the parameters
+	my %params = ();
+	$params{'directory'} = $self->{program_dir};
+	$params{'type'} = $self->{weka_type};
+	$params{'weka_size'} = $self->{weka_size};
+	$params{'features'} = $self->{features};
+	$params{'buckets'} = $self->{bucketsNum};
+	$params{'debug'} = $debug;
+	$modelboy = NanoB2B::NER::Modelman->new(\%params);			
+
+	for(my $a = $self->{fileIndex}; $a <= $totalTags; $a++){
+		$uniSub->printColorDebug("bold red", "##         MODEL FILE #$a / $totalTags         ##\n");
+		my $tag = $tags[$a - 1];
+		$modelboy->make_model_file($tag);
+		$uniSub->printColorDebug("bold red", "##         FINISHED MODEL #$a - $tag!        ##\n");
+		sleep(1);
+	}
+}
+
+
 =head3 avgSet
 
 description:
@@ -820,14 +944,6 @@ sub avgSet{
 	}
 }
 
-sub isArr{
-	my $self = shift;
-	my $arr_ref = shift;
-	my @arr = @$arr_ref;
-
-	print "yay\n";
-
-}
 
 #sorts a directory by the file size
 # input  : @files  <-- get the list of files in the folder
